#ifndef OSXSYSTEM_MAIN_H
#define OSXSYSTEM_MAIN_H

#include <AGL/AGL.h>
#include <OpenGL/OpenGL.h>
#include <OpenGL/gl.h>
#include <Carbon/Carbon.h>

#include <vector>
#include <string>

#include "system/system.h"
#include "math/vec.h"

//#define COCOA_INSTEAD_OF_CARBON

class OSXSystem : public PSystem {
public:
	OSXSystem() {reset(); }
	~OSXSystem() {shutdown(); }

	//external calls - to initialize, loop, and shutdown the application
	virtual void init();
	virtual void shutdown();
	
	virtual void loop(void (*loopCallback)());		
	virtual void requestExit();
	bool wasExitRequested() { return exitRequested; }
	
	virtual void setInitDoubleBuffered(bool _initDoubleBuffered) { initDoubleBuffered = _initDoubleBuffered; }
	virtual void setInitPos(const vec2i &_initPos) { initPos = _initPos; }
	virtual void setInitSize(const vec2i &_initSize) { initSize = _initSize; }
	
	virtual const std::string &getArg(int index) const { return args[index]; }
	virtual size_t getArgCount() const { return args.size(); }

	//this needs to be public 'cuz of the GLUT using methods as callbacks, etc
	void doLoop();

	//for all those who want to swap/update display buffers manually inter-render
	void updateDisplayBuffers();
	
	//up to the instanciator to fill / filled by InitBase
	std::vector<std::string> args;
		
#ifdef COCOA_INSTEAD_OF_CARBON
	CGLContextObj getCglCtx() { return cglCtx; }
#else
	AGLContext getAglCtx() { return aglCtx; }
#endif
	WindowRef getWnd() { return wnd; }

protected:
	void reset();

	//internal loop methods
	//for now I'm going to use a callback method... maybe switch it to an (abstract?) object later...
	void (*loopCallback)();
	void preUpdate();
	void postUpdate();
	
	bool initDoubleBuffered;
	bool initFullScreen;
	vec2i initPos;
	vec2i initSize;
	
	bool doubleBuffered;
	bool exitRequested;
#ifdef COCOA_INSTEAD_OF_CARBON
	CGLContextObj cglCtx;
#else
	AGLContext aglCtx;
#endif
	WindowRef wnd;
};

#endif	//OSXSYSTEM_MAIN_H
