#include <iostream>
#include <Carbon/Carbon.h>
#include <stdio.h>

#include "osxsystem/dialog.h"
#include "system/exception.h"

using namespace std;

//TODO - extensions are ignored atm

static char tohex(char ch) {
	if (ch >= '0' && ch <= '9') return ch - '0';
	if (ch >= 'a' && ch <= 'f') return ch - 'a' + 10;
	if (ch >= 'A' && ch <= 'F') return ch - 'A' + 10;
	return -1;
}

//convert in-place
//throws exceptions
static void urlToFile(char *fn) {
	const char *prefix = "file://localhost";
	int prefixlen = strlen(prefix);
	if (memcmp(fn, prefix, prefixlen)) throw Exception() << "expected filename " << fn << " to have a prefix " << prefix;
	strcpy(fn, fn + prefixlen);
	
	//next unescape any characters
	const char *src = fn;
	char *dst = fn;
	for (;*src;src++, dst++) {
		if (*src == '%') {
			src++;
			int hd = tohex(*src); src++;
			int ld = tohex(*src);
			*dst = (hd << 4) | ld;
		} else {
			*dst = *src;
		}
	}
}

#define APPLECHECKSTATUS(x)		if (osStatus = (x)) throw Exception() << #x << " got status " << osStatus
#define APPLECHECKERROR(x)		if (osErr = (x)) throw Exception() << #x << " got error " << osErr

bool OSXDialog::open(const std::string &ext, std::string &result) {
	char filename[4096];
	memset(filename, 0, sizeof(filename));

	bool success = false;
	try {
		OSStatus osStatus = 0;
		OSErr osErr = 0;
		
		NavDialogCreationOptions options;
		APPLECHECKSTATUS(NavGetDefaultDialogCreationOptions(&options));
		//fileOpenOptions.clientName = CFSTR("Choose File to Open...");
		
		NavDialogRef dialog;
		APPLECHECKSTATUS(NavCreateGetFileDialog(&options, NULL, NULL, NULL, NULL, NULL, &dialog));
		APPLECHECKSTATUS(NavDialogRun(dialog));

		NavReplyRecord reply;
		osStatus = NavDialogGetReply(dialog, &reply);
		if (osStatus != userCanceledErr) {
			if (osStatus) throw Exception() << "NavDialogGetReply got status " << osStatus;

			if (reply.validRecord) {
				
				FSRef fsRef;
				Size actualSize;
				APPLECHECKERROR(AEGetNthPtr(&reply.selection, 1, typeFSRef, NULL, NULL, &fsRef, sizeof(fsRef), &actualSize));
			
				CFURLRef urlRef = CFURLCreateFromFSRef(NULL, &fsRef);
				if (!urlRef) throw Exception() << "CFURLCreateFromFSRef got null";
				
				if (reply.saveFileName) {
					CFURLRef newRef = CFURLCreateCopyAppendingPathComponent(kCFAllocatorDefault, urlRef, reply.saveFileName, false);
					if (!newRef) throw Exception() << "CFURLCreateCopyAppendingPathComponent got null";
					CFRelease(urlRef);			//TODO - delete parentURLRef here
					urlRef = newRef;
				}
				
				CFStringRef stringRef = CFURLGetString(urlRef);
				
				//makes it this far now
				CFStringGetCString(stringRef, filename, sizeof(filename), kCFStringEncodingUTF8);
				//CFRelease(stringRef); //delete stringRef here
				CFRelease(urlRef); //delete urlRef here
				
				//now go from url to absolute path
				//first remove the prefix
				urlToFile(filename);

				success = true;

			}
			APPLECHECKERROR(NavDisposeReply(&reply));
		}
		
		NavDialogDispose(dialog);
	} catch (Exception &t) {
		cerr << "OSXDialog::open failed: " << t << "\n";
	}
	if (success) {
		result = filename;
	}
	return success;
}

bool OSXDialog::save(const std::string &ext, std::string &result) {
	char filename[4096];
	memset(filename, 0, sizeof(filename));

	bool success = false;
	try {
		OSStatus osStatus = 0;
		OSErr osErr = 0;
		
		NavDialogCreationOptions options;
		APPLECHECKSTATUS(NavGetDefaultDialogCreationOptions(&options));
		//fileOpenOptions.clientName = CFSTR("Choose File to Open...");
		
		NavDialogRef dialog;
		APPLECHECKSTATUS(NavCreatePutFileDialog(&options, 0, 0, NULL, NULL, &dialog));
		APPLECHECKSTATUS(NavDialogRun(dialog));

		NavReplyRecord reply;
		osStatus = NavDialogGetReply(dialog, &reply);
		if (osStatus != userCanceledErr) {
			if (osStatus) throw Exception() << "NavDialogGetReply got status " << osStatus;

			if (reply.validRecord) {
				
				FSRef fsRef;
				Size actualSize;
				APPLECHECKERROR(AEGetNthPtr(&reply.selection, 1, typeFSRef, NULL, NULL, &fsRef, sizeof(fsRef), &actualSize));
			
				CFURLRef urlRef = CFURLCreateFromFSRef(NULL, &fsRef);
				if (!urlRef) throw Exception() << "CFURLCreateFromFSRef got null";
				
				if (reply.saveFileName) {
					CFURLRef newRef = CFURLCreateCopyAppendingPathComponent(kCFAllocatorDefault, urlRef, reply.saveFileName, false);
					if (!newRef) throw Exception() << "CFURLCreateCopyAppendingPathComponent got null";
					CFRelease(urlRef);			//TODO - delete parentURLRef here
					urlRef = newRef;
				}
				
				CFStringRef stringRef = CFURLGetString(urlRef);
				
				//makes it this far now
				CFStringGetCString(stringRef, filename, sizeof(filename), kCFStringEncodingUTF8);
				//CFRelease(stringRef); //delete stringRef here
				CFRelease(urlRef); //delete urlRef here
				
				//now go from url to absolute path
				//first remove the prefix
				urlToFile(filename);

				success = true;

			}
			APPLECHECKERROR(NavDisposeReply(&reply));
		}
		
		NavDialogDispose(dialog);
	} catch (Exception &t) {
		cerr << "EditorFuncImpl::doSaveWorldDialog failed: " << t << "\n";
	}
	if (success) {
		result = filename;
	}
	return success;
}
